@@ -86,7 +86,7 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 	long long int all_bases = guess_gene_bases(chro_files,chro_file_number);
 	double local_begin_ftime = 0.;
 
-	int chro_table_maxsize=100;
+	int chro_table_maxsize=100, dump_res = 0;
 	unsigned int * read_offsets = malloc(sizeof(unsigned int) * chro_table_maxsize);
 	char * read_names = malloc(MAX_READ_NAME_LEN * chro_table_maxsize);
 	gehash_t table;
@@ -115,7 +115,8 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 		return -1;
 	}
 
-	if(gehash_create_ex(& table, segment_size, 0, SUBINDEX_VER2, GENE_SLIDING_STEP)) return 1;
+	int padding_around_contigs = MAX_READ_LENGTH;
+	if(gehash_create_ex(& table, segment_size, 0, SUBINDEX_VER2, GENE_SLIDING_STEP, padding_around_contigs)) return 1;
 
 	if(MARK_NONINFORMATIVE_SUBREADS)
 		copy_non_informative_subread(&table, huge_table);
@@ -124,7 +125,7 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 		if(gvindex_init(&value_array_index, 0)) return 1;
 
 	file_number = 0;
-	offset = 0;
+	offset = table.padding;
 	table_no = 0;
 	read_no = 0;
 
@@ -162,20 +163,18 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 					sprintf (fn, "%s.%02d.%c.tab", index_prefix, table_no, IS_COLOR_SPACE?'c':'b');
 					SUBREADfflush(stdout);
 
-					gehash_dump(&table, fn);
+					if(!dump_res)dump_res |= gehash_dump(&table, fn);
 
 					if(VALUE_ARRAY_INDEX)
 					{
 						sprintf (fn, "%s.%02d.%c.array", index_prefix, table_no, IS_COLOR_SPACE?'c':'b');
-						gvindex_dump(&value_array_index, fn);
+						if(!dump_res)dump_res |= gvindex_dump(&value_array_index, fn);
 						gvindex_destory(&value_array_index) ;
 					}
 
-
-
 					gehash_destory(&table);
 
-					read_offsets[read_no-1] = offset;
+					read_offsets[read_no-1] = offset + table.padding;
 
 					for(i=table_no+1; i<100; i++)
 					{
@@ -207,12 +206,17 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 
 				geinput_readline(&ginp, fn, 0);
 
-				if(read_no>0)
-					read_offsets[read_no-1] = offset;
+				if(offset==table.padding)
+					local_begin_ftime = miltime();
+
+				if(read_no>0){
+					read_offsets[read_no-1] = offset + table.padding;
+					offset += 2*table.padding;
+				}
 
 				//printf("TTTXT FN=%s\n",fn);
 
-				for(i=0;(fn[i+1] != ' ' && fn[i+1] != '\0' && fn[i+1] != '\t' && i<47); i++)
+				for(i=0;(fn[i+1] != ' ' && fn[i+1] != '\0' && fn[i+1] != '\t' && i<MAX_CHROMOSOME_NAME_LEN - 1); i++)
 					*(read_names + MAX_READ_NAME_LEN*read_no + i) = fn[i+1];
 
 				*(read_names + MAX_READ_NAME_LEN*read_no + i) = 0;
@@ -251,8 +255,6 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 				else
 					array_int_key = int_key = genekey2int(window,GENE_SPACE_BASE);
 	
-				if(offset==0)
-					local_begin_ftime = miltime();
 			}
 			if (status == FULL_PARTITION) 
 			{
@@ -276,11 +278,11 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 				sprintf(fn, "%s.%02d.%c.tab", index_prefix, table_no, IS_COLOR_SPACE?'c':'b');
 				SUBREADfflush(stdout);
 
-				gehash_dump(&table, fn);
+				if(!dump_res)dump_res |= gehash_dump(&table, fn);
 				if(VALUE_ARRAY_INDEX)
 				{
 					sprintf(fn, "%s.%02d.%c.array", index_prefix, table_no, IS_COLOR_SPACE?'c':'b');
-					gvindex_dump(&value_array_index, fn);
+					if(!dump_res)dump_res |= gvindex_dump(&value_array_index, fn);
 				}
 
 				table_no ++;
@@ -319,7 +321,7 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 				else
 					array_int_key = int_key = genekey2int(window, GENE_SPACE_BASE);
 				
-				if(gehash_create_ex(&table, segment_size, 0, SUBINDEX_VER2, GENE_SLIDING_STEP)) return 1;
+				if(gehash_create_ex(&table, segment_size, 0, SUBINDEX_VER2, GENE_SLIDING_STEP, padding_around_contigs)) return 1;
 				if(MARK_NONINFORMATIVE_SUBREADS)
 					copy_non_informative_subread(&table, huge_table);
 				if(VALUE_ARRAY_INDEX)
@@ -335,11 +337,12 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 				int is_no_info = gehash_exist(huge_table, int_key);
 				if(!is_no_info)
 				{
+					//SUBREADprintf("INSERT KEY=%u AT %u\n", int_key, offset);
 					if(gehash_insert(&table, int_key, offset - (IS_COLOR_SPACE?1:0))) return 1;
 				}
 				if(VALUE_ARRAY_INDEX)
 				{
-					gvindex_set(&value_array_index, offset - (IS_COLOR_SPACE?0:0), array_int_key);
+					gvindex_set(&value_array_index, offset - (IS_COLOR_SPACE?0:0), array_int_key, padding_around_contigs);
 				}
 			}
 
@@ -354,11 +357,14 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 				next_char = geinput_next_char(&ginp);
 				if(next_char < 0)
 				{
+					gvindex_set(&value_array_index, offset - (IS_COLOR_SPACE?0:0), array_int_key, padding_around_contigs);
+
 					if (next_char == -1) status = NEXT_READ;
 					if (next_char == -2) status = NEXT_FILE;
 					if (next_char == -3) return 0;
 					break;
 				}
+				//SUBREADprintf("NEXT_CH=%c\n", next_char);
 
 				if (next_char == 'N' )skips = 16;
 				else if (skips>0){
@@ -388,13 +394,15 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 					array_int_key = int_key;
 				}
 
-				if (offset>0 && offset % (all_bases / 12) == 0)
-				{
-					double finished_rate = offset*1.0/all_bases;
-					double base_per_second = offset / (miltime()-local_begin_ftime);
-					double ETA = (all_bases-offset) / base_per_second;
-					print_build_log(finished_rate,base_per_second,ETA, all_bases);
-					SUBREADfflush(stdout) ;
+				if(all_bases > 12){
+					if (offset>0 && offset % (all_bases / 12) == 0)
+					{
+						double finished_rate = offset*1.0/all_bases;
+						double base_per_second = offset / (miltime()-local_begin_ftime);
+						double ETA = (all_bases-offset) / base_per_second;
+						print_build_log(finished_rate,base_per_second,ETA, all_bases);
+						SUBREADfflush(stdout) ;
+					}
 				}
 
 				offset ++;
@@ -410,9 +418,27 @@ int build_gene_index(const char index_prefix [], char ** chro_files, int chro_fi
 
 		}
 	}
-	free(fn);
 	free(read_names);
 	free(read_offsets);
+	if(dump_res){
+		SUBREADprintf("No index was built.\n");
+		sprintf(fn, "%s.files", index_prefix);
+		unlink(fn);
+		sprintf(fn, "%s.reads", index_prefix);
+		unlink(fn);
+		int index_i;
+		for(index_i = 0; index_i <= 99; index_i++){
+			sprintf(fn, "%s.%02d.b.tab", index_prefix, index_i);
+			unlink(fn);
+			sprintf(fn, "%s.%02d.c.tab", index_prefix, index_i);
+			unlink(fn);
+			sprintf(fn, "%s.%02d.b.array", index_prefix, index_i);
+			unlink(fn);
+			sprintf(fn, "%s.%02d.c.array", index_prefix, index_i);
+			unlink(fn);
+		}
+	}
+	free(fn);
 	return 0;
 }
 
@@ -613,13 +639,15 @@ int scan_gene_index(const char index_prefix [], char ** chro_files, int chro_fil
 				read_len ++;
 
 
-				if (offset % (all_bases / 12) == 0)
-				{
-					double finished_rate = offset*1.0/all_bases;
-					double base_per_second = offset / (miltime()-local_begin_ftime);
-					double ETA = (all_bases-offset) / base_per_second;
-					print_build_log(finished_rate,base_per_second,ETA, all_bases);
-					SUBREADfflush(stdout) ;
+				if (all_bases>12){
+					if (offset % (all_bases / 12) == 0)
+					{
+						double finished_rate = offset*1.0/all_bases;
+						double base_per_second = offset / (miltime()-local_begin_ftime);
+						double ETA = (all_bases-offset) / base_per_second;
+						print_build_log(finished_rate,base_per_second,ETA, all_bases);
+						SUBREADfflush(stdout) ;
+					}
 				}
 
 				if(offset > 0xFFFFFFFDU)	
@@ -766,7 +794,7 @@ int check_and_convert_FastA(char ** input_fas, int fa_number, char * out_fa, uns
 	char * line_buf = malloc(MAX_READ_LENGTH);
 	char * read_head_buf = malloc(MAX_READ_LENGTH * 3);
 	unsigned int inp_file_no, line_no;
-	int written_chrs = 0;
+	int written_chrs = 0, is_disk_full = 0;
 	int chrom_lens_max_len = 100;
 	int chrom_lens_len = 0;
 	ERROR_FOUND_IN_FASTA = 0;
@@ -774,7 +802,7 @@ int check_and_convert_FastA(char ** input_fas, int fa_number, char * out_fa, uns
 
 	if(!out_fp)
 	{
-		SUBREADprintf("ERROR: The current directory is not writable, but the index builder needs to create temporary files in the current directory. Please change the working directory and rerun the index builder.\n");
+		SUBREADprintf("ERROR: the output directory is not writable, but the index builder needs to create temporary files in the current directory. Please change the working directory and rerun the index builder.\n");
 		return -1;
 	}
 
@@ -909,7 +937,13 @@ int check_and_convert_FastA(char ** input_fas, int fa_number, char * out_fa, uns
 
 				if(is_head_written)
 				{
-					fprintf(out_fp,"%s\n", line_buf);
+					int line_buf_len = strlen(line_buf);
+					int writen_len = fprintf(out_fp,"%s\n", line_buf);
+					if(writen_len < line_buf_len){
+						SUBREADprintf("ERROR: unable to write into the temporary file. Please check the free space of the output directory.\n");
+						is_disk_full = 1;
+						break;
+					}
 					(*chrom_lens)[chrom_lens_len-1] = read_len;
 					(*chrom_lens)[chrom_lens_len] = 0;
 				}
@@ -923,6 +957,7 @@ int check_and_convert_FastA(char ** input_fas, int fa_number, char * out_fa, uns
 		}
 
 		fclose(in_fp);
+		if(is_disk_full) break;
 	}
 
 
@@ -936,7 +971,7 @@ int check_and_convert_FastA(char ** input_fas, int fa_number, char * out_fa, uns
 		return 1;
 	}
 
-	if(is_repeated_chro)
+	if(is_repeated_chro|| is_disk_full)
 		return 1;
 
 	if(ERROR_FOUND_IN_FASTA)
@@ -1151,18 +1186,27 @@ int main_buildindex(int argc,char ** argv)
 		char * fasta_fn = *(argv+optind+x1);
 		int f_type = probe_file_type_fast(fasta_fn);
 		if(f_type != FILE_TYPE_FASTA && f_type != FILE_TYPE_NONEXIST){
-			SUBREADprintf("WARNING: '%s' is not a FASTA file.\n", fasta_fn);
+			SUBREADprintf("ERROR: '%s' is not a Fasta file.\n", fasta_fn);
 			if(f_type == FILE_TYPE_GZIP_FASTA){
-				SUBREADprintf("         The index builder does not accept gzipped files. The outcome is undefined.\n");
+				SUBREADprintf("The index builder does not accept gzipped Fasta files.\nPlease decompress the gzipped Fasza files before building the index.\n");
 			}
+			STANDALONE_exit(-1);
 		}
 	}
 
 
 	begin_ftime = miltime();
 
+	for(x1 = strlen(output_file); x1 >=0; x1--){
+		if(output_file[x1]=='/'){
+			memcpy(tmp_fa_file, output_file, x1);
+			tmp_fa_file[x1]=0;
+			break;
+		}
+	}
+	if(tmp_fa_file[0]==0)strcpy(tmp_fa_file, "./");
 
-	sprintf(tmp_fa_file, "./subread-index-sam-%06u-XXXXXX", getpid());
+	sprintf(tmp_fa_file+strlen(tmp_fa_file), "/subread-index-sam-%06u-XXXXXX", getpid());
 	mkstemp(tmp_fa_file);
 
 	sprintf(log_file_name, "%s.log", output_file);
@@ -1171,6 +1215,7 @@ int main_buildindex(int argc,char ** argv)
 	signal (SIGTERM, SIGINT_hook);
 	signal (SIGINT, SIGINT_hook);
 
+
 	int ret = check_and_convert_FastA(argv+optind , argc - optind, tmp_fa_file, &chromosome_lengths, log_fp, log_file_name);
 	if(log_fp)
 		fclose(log_fp);
